"""
main.py
Set up, connect the processors, and run a job. 

*Don't use this to start the project; use run.py for proper
directory perspective.*

Refer to readme.txt for an overview of the code.

"""
from processors import useragent, urldiff, mediaqueries
from util import logger
from time import time
import argparse
import os
import sys

#Processor switches
#Important: each processor assumes the previous one has created its
#files. This switch system is a crude (but functional) way of resuming
#interrupted jobs, so if something crashes in a later processor we
#don't have to start completely over. These switches are not intended
#for skipping parts of an analysis.
urldiff_on=True
useragent_on=True
mediaqueries_on=True

def main():
    #Input
    parser=argparse.ArgumentParser(description='System for finding mobile versions of web pages.',
                                   epilog='URLs will be read from stdin if no file is specified')
    
    parser.add_argument('-j', '--job-name',
                        help='name of job, output will be placed in jobs/[job name]')

    
    parser.add_argument('-f', '--file',
                        type=argparse.FileType('r'),
                        help='input file, should be a list of URLs')
    
    parser.add_argument('-q', '--quiet',
                        action='store_true',
                        help='do not write logging output to stderr')
    
    args=parser.parse_args()
    
    logger.stderr_enabled=not args.quiet
    
    if not useragent_on:
        print 'Warning: useragent processor disabled'
    if not urldiff_on:
        print 'Warning: urldiff processor disabled'
    
    job_name=args.job_name
    while not job_name:
        job_name=raw_input('Job name: ')
    
    #Load URLs
    urls=[]
    if args.file:
        #...from file
        for line in args.file:
            if line[0]=='#':
                continue
            urls.append(line.strip())
    else:
        #...from stdin
        while True:
            try:
                urls.append(raw_input())
            except EOFError:
                break
    
    #Set up output directory
    os.chdir(os.path.dirname(os.path.realpath(__file__)))    
            
    output_base=os.path.join(sys.path[0], 'jobs', job_name)
    
    try:
        os.mkdir(os.path.join(sys.path[0], 'jobs'))
    except:
        pass
    
    try:
        os.mkdir(output_base)
    except OSError as e:
        print e
        entry=raw_input('Job %s already exists, overwrite? (y/n) ' % job_name)
        
        if entry=='y':
            os.system('rm -fr %s' % output_base)
            os.mkdir(output_base)
            
    logger.log_file=open(os.path.join(output_base, 'log.txt'), 'a')
    time_file=open(os.path.join(output_base, 'time.txt'), 'a')
    
    #Processor connection and calling code
    
    #Processors tend to follow the pattern of:
    #1. Make dir and set up files
    #2. Call processor with files
    #3. Close up files
    
    #useragent processor
    if useragent_on:
        logger.log('beginning useragent processing')
        
        #Setup
        try:
            os.mkdir(os.path.join(output_base, 'useragent'))
        except OSError:
            entry=raw_input('useragent folder detected, overwrite? (y/n) ')
            if entry=='y':
                os.system('rm -fr %s' % os.path.join(output_base, 'useragent'))
                os.mkdir(os.path.join(output_base, 'useragent'))
        
        found_file=open(os.path.join(output_base, 'useragent', 'found.txt'), 'a')
        notfound_file=open(os.path.join(output_base, 'useragent', 'notfound.txt'), 'a')
        stats_file=open(os.path.join(output_base, 'useragent', 'stats.txt'), 'a')
        error_file=open(os.path.join(output_base, 'useragent', 'error.txt'), 'a')
        
        #Call processor
        start=time()
        
        useragent.find(urls, found_file, notfound_file, stats_file, error_file)
        
        elapsed=time()-start
        time_file.write('useragent %f\n' % elapsed)
        time_file.flush()
        
        #Clean up
        found_file.close()
        notfound_file.close()
        stats_file.close()
        error_file.close()
        
        logger.log('useragent done, elapsed time %fs' % elapsed)
    
    #urldiff processor
    if urldiff_on:
        logger.log('beginning urldiff processing')
        
        #Setup
        try:
            os.mkdir(os.path.join(output_base, 'urldiff'))
        except OSError:
            entry=raw_input('urldiff folder detected, overwrite? (y/n) ')
            if entry=='y':
                os.system('rm -fr %s' % os.path.join(output_base, 'urldiff'))
                os.mkdir(os.path.join(output_base, 'urldiff'))
        
        urls_file=open(os.path.join(output_base, 'useragent', 'notfound.txt'), 'r')
        examples_file=open(os.path.join(output_base, 'useragent', 'found.txt'), 'r')    
        found_file=open(os.path.join(output_base, 'urldiff', 'found.txt'), 'a')
        notfound_file=open(os.path.join(output_base, 'urldiff', 'notfound.txt'), 'a')
        stats_file=open(os.path.join(output_base, 'urldiff', 'stats.txt'), 'a')
        error_file=open(os.path.join(output_base, 'urldiff', 'error.txt'), 'a')
        
        #Call processor
        start=time()
    
        urldiff.find(urls_file, examples_file,
                     found_file, notfound_file,
                     stats_file, error_file)
        
        elapsed=time()-start
        time_file.write('urldiff %f\n' % elapsed)
        time_file.flush()
        
        #Clean up
        urls_file.close()
        examples_file.close()
        found_file.close()
        notfound_file.close()
        stats_file.close()
        error_file.close()
        
        logger.log('urldiff done, elapsed time %fs' % elapsed)
        
    #mediaqueries processor
    if mediaqueries_on:
        logger.log('beginning mediaqueries processing')
        
        #Setup
        try:
            os.mkdir(os.path.join(output_base, 'mediaqueries'))
        except OSError:
            entry=raw_input('mediaqueries folder detected, overwrite? (y/n) ')
            if entry=='y':
                os.system('rm -fr %s' % os.path.join(output_base, 'mediaqueries'))
                os.mkdir(os.path.join(output_base, 'mediaqueries'))
        
        urls_file=open(os.path.join(output_base, 'urldiff', 'notfound.txt'), 'r')
        found_file=open(os.path.join(output_base, 'mediaqueries', 'found.txt'), 'a')
        notfound_file=open(os.path.join(output_base, 'mediaqueries', 'notfound.txt'), 'a')
        stats_file=open(os.path.join(output_base, 'mediaqueries', 'stats.txt'), 'a')
        error_file=open(os.path.join(output_base, 'mediaqueries', 'error.txt'), 'a')
        
        #Call processor
        start=time()
    
        mediaqueries.find(urls_file, found_file, notfound_file, stats_file, error_file)
        
        elapsed=time()-start
        time_file.write('mediaqueries %f\n' % elapsed)
        time_file.flush()
        
        #Clean up
        urls_file.close()
        found_file.close()
        notfound_file.close()
        stats_file.close()
        error_file.close()
        
        logger.log('mediaqueries done, elapsed time %fs' % elapsed)
    
    #Finish up
    logger.log('finishing up')
    #Combine found.txt file into a root found.txt
    os.system('cat %s %s %s > %s' % (os.path.join(output_base, 'useragent', 'found.txt'),
                                    os.path.join(output_base, 'urldiff', 'found.txt'),
                                    os.path.join(output_base, 'mediaqueries', 'found.txt'),
                                    os.path.join(output_base, 'found.txt')))
    
    #Copy mediaquueries' notfound.txt to root,
    #as it represents the final URLS not found
    os.system('cp %s %s' % (os.path.join(output_base, 'mediaqueries', 'notfound.txt'),
                            os.path.join(output_base, 'notfound.txt')))
    
    logger.log('done')
