"""
urldiff.py - urldiff processor

Attempts to find mobile versions of pages by looking at previously
found pages and inferring the transformations from desktop urls to
mobile urls. Generates several url guesses and tests them out. If one
of them is found to be valid, it is tested to see if it is significantly
different from the desktop page before being recorded as a mobile site.

This implementation relies heavily on Google's diff-match-patch library:
http://code.google.com/p/google-diff-match-patch/

This also makes use of Ben Hoyt's soft 404 detection script:
http://blog.brush.co.nz/2008/01/soft404s/

"""
import diff_match_patch
import soft404
from ..util.logger import log
from ..util.confirm_diff import confirm_diff, get_last_score
from ..util.record import record_found, record_notfound
import urllib2
import socket

#Limit URL guesses to those with probability value (decimal percent) 
#over the threshold when all the patches are applied to a URL and the
#results are tabulated (see get_guesses).
#This keeps mistaken inferences from messing with the results.
threshold=.015

#Object to interact with the diff_match_patch library
dmp=diff_match_patch.diff_match_patch()

def find(urls, pattern_examples, found_file, notfound_file,
         stats_file, error_file):
    #Make patches
    patches=[]
    for line in pattern_examples:
        url, mobile_url=line.strip().split(' ', 1)
        
        if url!=mobile_url:
            #print 'patching',url,mobile_url
            patches.append(dmp.patch_make(url, mobile_url))
    
    #Print example transformations
    log('\tpatterns generated:')
    pattern_examples=get_guesses('http://www.example.edu/', patches)
    pattern_examples=sorted(pattern_examples, key=len)
    for pattern_example in pattern_examples:
        log('\t\t%s' % pattern_example)
    
    #Guess for all the urls
    for url in urls:
        url=url.strip()
        
        #Append a trailing slash if there's not one, so it plays
        #nicely with the patches
        if url[-1]!='/':
            url+='/'
                
        log('urldiff find %s' % url)
        
        #Guess based on patches
        mobile_urls=get_guesses(url, patches)
        found=False
        for mobile_url in sorted(mobile_urls, key=len):
            if url==mobile_url:
                continue
            
            log('\ttrying %s' % mobile_url)
            try:
                #Poke the URL (this will throw an exception if the URL
                #is invalid, which happens most of the time)            
                urllib2.urlopen(mobile_url, timeout=20)
                
                #Check for soft 404s
                log('\tchecking for soft 404... ', False)

                if soft404.is_dead(mobile_url):
                    log('soft 404 detected')
                    continue
                else:
                    log('not detected')
                
                #If we've got through the above stuff, we can assume
                #the url at least works, so let's see if it's different
                #from the fixed page
                if not confirm_diff(url, mobile_url):
                    continue
                
                record_found(url, mobile_url, get_last_score(),
                             found_file, stats_file)
                found=True
                break
            
            #Thrown if the guessed url is invalid (quite likely)
            except urllib2.URLError:
                pass
            except urllib2.HTTPError:
                pass
            except socket.error:
                pass
            except socket.timeout:
                log('\t%s timeout' % mobile_url)
                error_file.write('%s timeout' % mobile_url)
            except Exception as e:
                #Some exception we haven't checked for above
                #Probably should find out what it is
                error_file.write('%s %s' % (mobile_url, str(type(e))))
            
        if not found:
            record_notfound(url, get_last_score(),
                            notfound_file, stats_file)

def get_guesses(fixed_url, patches):
    """Generate guess URLs from the set of patches."""
    #Apply all patches to the URL and tabulate the results
    #Many patches will result in the same URL
    #We count these up so we can find the major ones and ignore the
    #insignificant/broken URLs
    url_count={}
    for patch in patches:
        mobile_url, result=dmp.patch_apply(patch, fixed_url)
        
        #Make sure the patch was successful
        success=True
        for r in result:
            if r==False:
                success=False
                break
        
        if success:            
            url_count[mobile_url]=url_count.get(mobile_url, 0)+1
    
    #Normalize values
    total=sum(url_count.values())
    try:
        factor=1.0/total
    except ZeroDivisionError:
        return []
    
    url_p={}
    for mobile_url, count in url_count.iteritems():
        url_p[mobile_url]=count*factor
    
    #Go through the guesses and take the ones with a probability over
    #the defined threshold
    mobile_urls=set()
    for mobile_url, p in url_p.iteritems():
        if p>threshold and url_count[mobile_url]>1:
            mobile_urls.add(mobile_url)
    
    return mobile_urls
