"""
useragent.py - useragnet processor

Attempts to find a mobile version of a given URL by requesting with
different user agents.

Notes:
    * Only finds mobile pages given automatically through user agents.
    * Does not work properly with window.location=[url] redirects
      (this is a PhantomJS issue). Often it detects the presence of a
      mobile page, but not the actual URL.

How it works:
    1. Request the URL with fixed and mobile user agents.
    2. If the mobile request ends up at a different URL than the fixed
       request, assume the mobile URL is a mobile version of the page.
    3. Calculate the tag frequency distribution analysis (TFDA) score
       for the fixed and mobile requests (see tfda.py).
    4. If the score is over 0 (indicating a difference), request copies
       of the fixed version and calculate the mean and standard
       deviation of TFDA scores between the copies and the original.
    5. If the TFDA score of the mobile request is two standard
       deviations above the mean score of the fixed request copies,
       consider the difference significant. This means the URL itself
       has a mobile version (instead of providing a redirect).

"""
from ..util.request import request
from ..util.logger import log
from ..util.confirm_diff import confirm_diff, get_last_score
from ..util.record import record_found, record_notfound

def find(urls, found_file, notfound_file, stats_file, error_file):
    for url in urls:
        log('useragent find %s' % url)
        
        #Request fixed and mobile versions of the page
        log('\trequesting fixed... ', False)
        fixed_status, fixed_url, fixed_stylesheets, fixed_content=request('fixed', url, True)
        log('mobile...')
        mobile_status, mobile_url, mobile_stylesheets, mobile_content=request('mobile', url, True)
        
        #If we've gotten errors in the request, skip the URL
        if not fixed_status==mobile_status=='success':
            log('\terror: fixed %s, mobile %s' % (fixed_status, mobile_status))
            error_file.write('%s %s %s %s %s\n' % (url, fixed_url, fixed_status,
                                                   mobile_url, mobile_status))
            continue
        
        #If the final URLs are different, we've probably been
        #redirected to a mobile site
        if fixed_url!=mobile_url:
            record_found(fixed_url, mobile_url, 'redirect',
                         found_file, stats_file)
            continue
        
        #If stylesheets are different, they probably have mobile CSS
        fixed_ss_set=set(fixed_stylesheets)
        mobile_ss_set=set(mobile_stylesheets)
        ss_diff=fixed_ss_set.difference(mobile_ss_set)
        ss_diff.update(mobile_ss_set.difference(fixed_ss_set))
        if ss_diff:
            log('\tstylesheets difference')
            record_found(fixed_url, mobile_url, 'stylesheets',
                         found_file, stats_file)
            continue
        
        #Confirm there's a significant difference between the versions
        if confirm_diff(fixed_url, mobile_url, fixed_content, mobile_content):
            record_found(fixed_url, mobile_url, get_last_score(),
                         found_file, stats_file)
        else:
            record_notfound(url, get_last_score(),
                            notfound_file, stats_file)
            